'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (from, to, args, config) {
	var params = void 0;
	try {
		params = getParams(from, to, args, config);

		getBumpActions().forEach(function (action) {
			action(params);
		});
	} catch (e) {
		_logger2.default.error(e.toString());
	}

	afterBump(params);
};

var _escapeStringRegexp = require('escape-string-regexp');

var _escapeStringRegexp2 = _interopRequireDefault(_escapeStringRegexp);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _fileHeader = require('../utils/file-header');

var _fileHeader2 = _interopRequireDefault(_fileHeader);

var _git = require('../utils/git');

var _git2 = _interopRequireDefault(_git);

var _logger = require('../utils/logger');

var _logger2 = _interopRequireDefault(_logger);

var _readme = require('../utils/readme');

var _readme2 = _interopRequireDefault(_readme);

var _svn = require('../utils/svn');

var _svn2 = _interopRequireDefault(_svn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Internal dependencies
 */
/**
 * External dependencies
 */
function getParams(from, to, args, config) {
	var release = args.release;
	var type = args.type || config.type || 'plugin';
	var files = config.files || {};
	var main = args.file || files.main;
	var gh = config.gh || {};
	var branch = args.branch || gh.branch || 'master';
	var originalBranch = _git2.default.getCurrentBranch();
	var wporg = config.wporg || {};
	var slug = wporg.slug;
	var username = wporg.username;
	var buildPath = wporg.buildPath;

	return {
		type: type,
		main: main,
		from: from,
		to: to,
		release: release,
		branch: branch,
		originalBranch: originalBranch,
		slug: slug,
		username: username,
		buildPath: buildPath
	};
}

function getBumpActions() {
	return [gitStash, switchGitBranch, validateParams, updateReadme, updateMainFile, gitCommit, gitPush, wporgDeploy];
}

function gitStash() {
	_logger2.default.info('Save current changes.');
	_git2.default.stash();
}

function switchGitBranch(params) {
	_logger2.default.info('Switch to branch ' + params.branch + '.');
	_git2.default.checkout(params.branch);
}

function validateParams(params) {
	var from = params.from,
	    to = params.to;


	if (!_semver2.default.valid(semverit(from))) {
		throw new Error('Value <from>, ' + from + ', is not a valid version.');
	}

	if (!_semver2.default.valid(semverit(to))) {
		throw new Error('Value <to>, ' + to + ', is not a valid version.');
	}

	if (!_semver2.default.gte(semverit(to), semverit(from))) {
		throw new Error('Version of <to>, ' + to + ', must be greater than or equal to <from>, ' + from + '.');
	}
}

function updateMinVersion(content, oldVersion, newVersion) {
	return content.replace(new RegExp('(Requires at least\\s*:\\s*)' + (0, _escapeStringRegexp2.default)(oldVersion)), versionReplacer(newVersion));
}

function updateMaxVersion(content, oldVersion, newVersion) {
	return content.replace(new RegExp('(Tested up to\\s*:\\s*)' + (0, _escapeStringRegexp2.default)(oldVersion)), versionReplacer(newVersion));
}

function versionReplacer(newVersion) {
	return function (match, prefix) {
		return prefix + newVersion;
	};
}

function updateReadme(params) {
	var content = _readme2.default.getContent();
	var from = params.from,
	    to = params.to;

	var _readme$parse = _readme2.default.parse(content),
	    meta = _readme$parse.meta;

	var cFrom = meta['Requires at least'];
	var cTo = meta['Tested up to'];
	var updateFrom = cFrom && _semver2.default.neq(semverit(cFrom), semverit(from));
	var updateTo = cTo && _semver2.default.neq(semverit(cTo), semverit(to));

	if (updateFrom) {
		content = updateMinVersion(content, cFrom, from);
	}
	if (updateTo) {
		content = updateMaxVersion(content, cTo, to);
	}

	if (!updateFrom && !updateTo) {
		_logger2.default.warning('Nothing updated in readme.txt.');
	} else {
		_readme2.default.updateContent(content);
		_logger2.default.success('Updated readme.txt.');
	}
}

function semverit(v) {
	v = v.toString();

	var arr = v.split('.');
	switch (arr.length) {
		case 1:
			return arr[0] + '.0.0';
		case 2:
			return arr[0] + '.' + arr[1] + '.0';
		case 3:
			return v;
		default:
			return '';
	}
}

function updateMainFile(params) {
	var type = params.type,
	    main = params.main,
	    from = params.from,
	    to = params.to;

	var data = _fileHeader2.default.getFileData(main, type, {
		'Requires at least': 'Requires at least',
		'Tested up to': 'Tested up to'
	});
	var cFrom = data['Requires at least'];
	var cTo = data['Tested up to'];
	var updateFrom = cFrom && _semver2.default.neq(semverit(cFrom), semverit(from));
	var updateTo = cTo && _semver2.default.neq(semverit(cTo), semverit(to));

	var content = _fsExtra2.default.readFileSync(main, 'utf8');

	if (updateFrom) {
		content = updateMinVersion(content, cFrom, from);
	}
	if (updateTo) {
		content = updateMaxVersion(content, cTo, to);
	}

	if (!updateFrom && !updateTo) {
		_logger2.default.warning('Nothing updated in ' + main + '.');
	} else {
		_fsExtra2.default.outputFileSync(main, content, 'utf8');
		_logger2.default.success('Updated ' + main + '.');
	}
}

function getBumpMessage(from, to) {
	return 'Bump "Requires at least" to ' + from + ' and "Tested up to" to ' + to + '.';
}

function gitCommit(params) {
	var release = params.release,
	    from = params.from,
	    to = params.to;


	if (!release) {
		return _logger2.default.warning('Skip commit.');
	}

	try {
		_git2.default.add();
		_git2.default.commit(getBumpMessage(from, to));
		_logger2.default.success('Changes committed.');
	} catch (e) {}
}

function gitPush(params) {
	var release = params.release,
	    branch = params.branch;


	if (!release) {
		return _logger2.default.warning('Skip push.');
	}

	_logger2.default.info('Push changes to remote origin.');
	_git2.default.push(branch);
}

function wporgDeploy(params) {
	var from = params.from,
	    to = params.to,
	    release = params.release,
	    slug = params.slug,
	    username = params.username,
	    buildPath = params.buildPath;

	if (!release) {
		return _logger2.default.warning('Skip release.');
	}

	_fsExtra2.default.removeSync(buildPath);

	var root = (0, _path.resolve)(buildPath);
	var trunk = (0, _path.resolve)(buildPath, 'trunk');

	_svn2.default.checkout(_svn2.default.getWpOrgSvn(slug), root);
	_svn2.default.update(trunk, '--set-depth', 'infinity');

	// Only copy readme.txt to bump the change on wporg page.
	['readme.txt', 'README.txt'].forEach(function (r) {
		if (_fsExtra2.default.existsSync(r)) {
			_fsExtra2.default.copySync(r, (0, _path.resolve)(trunk, r));
		}
	});

	// The readme.txt is assumed to be tracked by svn already, so it's matter
	// of commit to trunk.
	_svn2.default.commit(trunk, username, getBumpMessage(from, to));
	_logger2.default.success('Committed to wp.org.');
}

function afterBump(params) {
	try {
		_logger2.default.info('Switch back to ' + params.originalBranch + '.');
		_git2.default.checkout(params.originalBranch);
		_logger2.default.info('Pop saved changes.');
		_git2.default.unstash();
		_fsExtra2.default.removeSync((0, _path.resolve)(process.cwd(), params.buildPath));
	} catch (e) {}
}
module.exports = exports['default'];