'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _archiver = require('archiver');

var _archiver2 = _interopRequireDefault(_archiver);

var _path = require('path');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _globAll = require('glob-all');

var _globAll2 = _interopRequireDefault(_globAll);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _read = require('read');

var _read2 = _interopRequireDefault(_read);

var _temp = require('temp');

var _temp2 = _interopRequireDefault(_temp);

var _logger = require('../utils/logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * External dependencies
 */
exports.default = function (args, config) {
	try {
		var params = getParams(args, config);

		prepareSource(params);
		generate(params);
	} catch (e) {
		_logger2.default.error(e.toString());
	}
};

/**
 * Internal dependencies
 */


function getParams(args, config) {
	var cwd = process.cwd();
	var wporg = config.wporg || {};
	var slug = wporg.slug || (0, _path.basename)(cwd);
	var src = _temp2.default.mkdirSync(slug);
	var dst = getDestination(args, config);
	var sourceFiles = getSourceFiles(config);

	return {
		src: src,
		dst: dst,
		sourceFiles: sourceFiles,
		slug: slug
	};
}

function getDestination(args, config) {
	var cwd = process.cwd();
	var wporg = config.wporg || {};
	var slug = wporg.slug || (0, _path.basename)(cwd);
	var _dst = args.output || (0, _path.resolve)(cwd, slug + '.zip');

	var dst = '~' === _dst.charAt(0) ? (0, _path.join)(_os2.default.homedir(), _dst.substr(1)) : _dst;

	// In case dst does not exist, make sure basedir exists, otherwise throws
	// error.
	if (!_fsExtra2.default.existsSync((0, _path.dirname)(dst))) {
		throw new Error((0, _path.dirname)(dst) + ' does not exist.');
	}

	// Make sure dst is not a directory that exists already.
	try {
		var st = _fsExtra2.default.lstatSync(dst);

		if (st.isDirectory()) {
			dst = (0, _path.resolve)(dst, slug + '.zip');
		}
	} catch (e) {}

	return dst;
}

function getSourceFiles(config) {
	var files = config.files || {};
	var src = files.src;
	var assets = _globAll2.default.sync(files.assets || []);

	if (!src) {
		throw new Error('Missing files to build. Specify it via `files.src` in .wpt.yml.');
	}

	if (!assets) {
		throw new Error('Missing assets for wp.org. Specify it via `files.assets` in .wpt.yml.');
	}

	return _globAll2.default.sync(src).filter(filterNotAssets(assets));
}

function prepareSource(params) {
	var copyToSrc = copyFileCallback(params.src);
	params.sourceFiles.forEach(copyToSrc);
}

function copyFileCallback(dst) {
	return function (filepath) {
		var stat = _fsExtra2.default.statSync(filepath);
		if (stat.isDirectory()) {
			_fsExtra2.default.ensureDir((0, _path.resolve)(dst, filepath));
		} else if (stat.isFile()) {
			_fsExtra2.default.copySync(filepath, (0, _path.resolve)(dst, filepath));
		}
	};
}

function filterNotAssets(assets) {
	return function (filepath) {
		var isAsset = false;
		for (var i = 0; i < assets.length; i++) {
			isAsset = _fsExtra2.default.statSync(filepath).isFile() && assets[i] === filepath;
			if (isAsset) {
				break;
			}
		}

		return !isAsset;
	};
}

function generate(params) {
	if (_fsExtra2.default.existsSync(params.dst)) {
		(0, _read2.default)({ prompt: 'Overwrite ' + params.dst + '?', 'default': 'yes' }, overwriteCallback(params));
	} else {
		write(params);
	}
}

function overwriteCallback(params) {
	return function (err, ok) {
		if (err) {
			return _logger2.default.error(err.message);
		}
		var cancel = !ok || 'y' !== ok.toLowerCase().charAt(0);

		return cancel ? _logger2.default.warning('Aborted.') : write(params);
	};
}

function write(params) {
	var out = _fsExtra2.default.createWriteStream(params.dst);
	var zip = (0, _archiver2.default)('zip');

	out.on('close', function () {
		_logger2.default.success('Written to ' + params.dst + '.');
		_fsExtra2.default.removeSync(params.src);
	});

	zip.on('error', function (err) {
		_logger2.default.error(err);
		_fsExtra2.default.removeSync(params.src);
	});

	zip.directory(params.src, params.slug);
	zip.pipe(out);
	zip.finalize();
}
module.exports = exports['default'];